@@ -1,21 +1,28 @@
 
-from pathlib import Path
+import io
+import json
+import linecache
+import os
+import pkg_resources
+import re
+import shutil
+import sys
+import time
 import traceback
+from pathlib import Path
 from contextlib import contextmanager
 from functools import wraps
-import pkg_resources
-import os
-import shutil
-from .pluginutils import Config as C, EndOfPlugin, PluginError, ExcludeFile, exclude, pluginError, endOfTemplate, invokeCmd
-from tempiny import Tempiny
 from itertools import accumulate, chain
-import io
-import click
-
 from traceback import print_exc
 
-import time
-import json
+from tempiny import Tempiny
+from .pluginutils import Config as C, EndOfPlugin, PluginError, ExcludeFile, exclude, pluginError, EndOfTemplate, endOfTemplate, invokeCmd, OptionParser
+
+import click
+
+
+
+sys.excepthook = print_exc
 
 APP = 'skbs'
 
@@ -39,11 +46,20 @@ class Include(object):
       raise FileNotFoundError(path_str)
 
     out_path, is_opt, is_template = Backend.parseFilePath(p, self.file_name_parser, None)
+    
+    if is_template != False :
+      with p.open('r') as in_f :
+        tempiny, _ = tempinyFromIterable(in_f)
+      is_template = is_template or (tempiny is not None)
 
     if is_opt :
       raise RuntimeError("An included file can't be optionnal")
     if is_template :
-      tempiny = Backend.getFirstMatch(self.tempiny_l, out_path)
+      if tempiny is None :
+        if out_path :
+          tempiny = Backend.getFirstMatch(self.tempiny_l, out_path)
+        else:
+          _, tempiny = self.tempiny_l[0]
       _locals = C(**self.base_locals, **_locals)
       out = io.StringIO()
       with p.open('r') as in_f :
@@ -59,7 +75,7 @@ class Include(object):
     else:
       val = p.read_text()
     # On unix-like system, a line must end with '\n'.
-    # Since the included concent could be in the middle of a line, the extra '\n' should be removed
+    # Since the included content could be in the middle of a line, the extra '\n' should be removed
     if val[-1] == '\n' :
       return val[:-1] 
     return val
@@ -76,39 +92,78 @@ class FileNameParser(object):
     if dir_template_filename is not None :
       self.dir_template_filename = dir_template_filename
     else :
-      self.dir_template_filename = template_prefix
+      self.dir_template_filename = template_prefix[0]
       if self.dir_template_filename is None :
         self.dir_template_filename = Backend.TEMPLATE_PREFIX
         
 
   def parse(self, name:str):
-    is_opt = False
-    is_raw = False
-    # See the Karnaugh table
-    if self.opt_prefix :
-      if is_opt := name.startswith(self.opt_prefix) :
-        name = name[len(self.opt_prefix):]
-      elif self.force_prefix :
-        if name.startswith(self.force_prefix) :
-          name = name[len(self.force_prefix):]
-    elif self.force_prefix :
-      if not (is_opt := (not name.startswith(self.force_prefix))) :
-        name = name[len(self.force_prefix):]
-      
-      
-    if self.raw_prefix :
-      if is_raw := name.startswith(self.raw_prefix) :
-        name = name[len(self.raw_prefix):]
-      elif self.template_prefix :
-        if name.startswith(self.template_prefix) :
-          name = name[len(self.template_prefix):]
-    elif self.template_prefix :
-      if not (is_raw := (not name.startswith(self.template_prefix))) :
-        name = name[len(self.template_prefix):]
+    """
+    Formula is as follow : 
+    O : opt prefix is defined
+    F : force prefix is defined
+    R : raw prefix is defined
+    T : template prefix is defined
 
-    is_template = not is_raw
+    o : opt prefex is present in filename
+    f : force prefex is present in filename
+    r : raw prefex is present in filename
+    t : template prefex is present in filename
+
+    h : a synheader line is present at file start
+    d : is_opt is defined
+    v : value of not is_opt
+
+    is_template = !r.t + t.!R.T + !r.R.!T + h(r.t.!R.!T + !r.!t.!R.!T + !r.!t.R.T)
+    
+    D = d.is_template
+
+    force = D.v + !D(!o.f + !o.F + f.!O + !o.O)
+
+    is_opt = !force
+
+    This function returns (name, _is_opt, _is_template)
+    where :
+
+    name : name with prefixed removed
+    _is_opt = !(!o.f + !o.F + f.!O + !o.O)
+    _is_template = {True if !r.t + t.!R.T + !r.R.!T ; False if !(!r.t + t.!R.T + !r.R.!T).!(r.t.!R.!T + !r.!t.!R.!T + !r.!t.R.T); None else }
+
+    """
+    Opre, O = self.opt_prefix
+    Fpre, F = self.force_prefix
+    Rpre, R = self.raw_prefix
+    Tpre, T = self.template_prefix
+    
+    o = False
+    if name.startswith(Opre) :
+      o = True
+      name = name[len(Opre):]
       
-    return name, is_opt, is_template
+    f = False
+    if name.startswith(Fpre) :
+      f = True
+      name = name[len(Fpre):]
+      
+    r = False
+    if name.startswith(Rpre) :
+      r = True
+      name = name[len(Rpre):]
+      
+    t = False
+    if name.startswith(Tpre) :
+      t = True
+      name = name[len(Tpre):]
+
+    Ea = t and ( not r or r and not R and T ) or not r and R and not T
+    Eb = (
+          r and     t and not R and not T or
+      not r and not t and ( not R and not T or R and T )
+    )
+
+    Ec = f and not O or not o and ( f or not F or O)
+
+    return name, not Ec, True if Ea else False if not Ea and not Eb else None
     
   
   def __call__(self, p:Path):
@@ -125,14 +180,14 @@ class OutStream(object):
   class PlaceholderWithKeepOnlySections(RuntimeError):
     pass
 
+  class MatchWithZeroLines(RuntimeError):
+    pass
+
   @staticmethod
-  def matcher(m_lines, begin):
-    if begin :
-      def match(lines, i):
-        return lines[i:i+len(m_lines)] == m_lines
-    else :
-      def match(lines, i):
-        return lines[i-len(m_lines):i] == m_lines
+  def matcher(m_lines, offset=0):
+    def match(lines, i):
+      i += offset
+      return lines[i:i+len(m_lines)] == m_lines
     return match
 
   class Section(object):
@@ -156,19 +211,19 @@ class OutStream(object):
     """
     
     """
-    def __init__(self, name, f, begin_line):
+    def __init__(self, name, cb, begin_line):
       self.name = name
-      self.f = f
+      self.cb = cb
       self.begin_line = begin_line
       self.ori_end_line = begin_line
       self.ori_begin_line = None
       self.ori_end_line = None
       self.sections = []
-      self.begin = f
+      self.begin = cb
       self.end = lambda *args, **kwargs:True
 
     def __repr__(self):
-      return f'Placeholder(name={self.name}, f={self.f}, bl={self.begin_line}, o_bl={self.ori_begin_line}, n_sec={len(self.sections)})'
+      return f'Placeholder(name={self.name}, cb={self.cb}, bl={self.begin_line}, o_bl={self.ori_begin_line}, n_sec={len(self.sections)})'
       
   def __init__(self):
     self.lines = []
@@ -177,48 +232,73 @@ class OutStream(object):
     self._placeholders = []
     self.placeholders = []
 
-  def beginSection(self, n=1, f=None, placeholder=None, overwrite=True):
+  def beginSection(self, m=0, n=1, cb=None, placeholder=None, overwrite=None):
     """
     Start an overwritten section.
-    @param f : A calback function ``f(lines, i)`` where ``lines`` is a list of the lines in the original file. The function should return true if it matches.
-    @param n : If ``f`` is None, Then the ``n`` following lines in the "virtually" outputed template (as if it were run for the first time) will be the line to match exactly in the original file to tag the section start.
+    
+    @param cb : A calback function ``cb(lines, i)`` where ``lines`` is a list of the lines in the original file, and ``i`` is the index of the current line being checked. The function should return true if lines[i] is the first line inside the section.
+    @param n
+    @param m : If ``cb`` is None, Then the section begin will try to match ``lines[cur + m : cur + n]`` in both the virtual output template and the original file.
     @param placeholder If a placeholder is specified and the original file does not have this section, then it will be put just before the placeholder (so that further added sections go always to the end)
+    @param overwrite : is True, use the content from the tempalte's virtual output, else, use the content of the original file.
+If left unset or ``None``, will have ``not keep_only_sections`` as value to get an intuitive behavior
+It can also be to a callable object with this signature : ``overwrite(original: list[str], virtual: list[str], ctx: Callable[[], 'Context']) -> list[str]``. Where ``origninal`` and ``virtual`` are respectively the content of the matched section. ctx is a callable to obtain the context, it returns an object with the follwing attributes read-only:
+
+ * ``o`` : Original file :
+    * ``o.lines`` : original file lines
+    * ``o.sec_b`` : begin of section
+    * ``o.sec_e`` : end of section
+ * ``v`` Virtual output :
+    * ``o.lines`` : original file lines
+    * ``o.sec_b`` : begin of section
+    * ``o.sec_e`` : end of section
+ * ``keep_only_sections``
     """
+    if m == n :
+      raise self.MatchWithZeroLines('A placeholder was create with n == 0')
     if self.cur_section is not None :
       raise OutStream.AlreadyInSectionError("Sections cannot be nested")
-    self.cur_section = OutStream.Section(placeholder, overwrite), len(self.lines), n, f
+    self.cur_section = OutStream.Section(placeholder, overwrite), len(self.lines), m, n, cb
 
 
-  def endSection(self, n=1, f=None):
+  def endSection(self, m=-1, n=0, cb=None):
     """
     End an overwritten section.
-    @param f : ``f`` is a calback function ``f(lines, i)`` where ``lines`` is a list of the lines in the original file. The function should return true if it matches.
-    @param n : If ``f`` is None, Then the ``n`` previous lines in the "virtually" outputed template (as if it were run for the first time) will be the line to match exactly in the original file to tag the section end.
+    
+    @param cb : ``cb`` is a calback function ``cb(lines, i)`` where ``lines`` is a list of the lines in the original file, and ``i`` is the index of the current line being checked. The function should return true if lines[i] is the last line (inclusive) inside the section.
+    @param n
+    @param m : If ``cb`` is None, Then the section end will try to match ``lines[cur + m : cur + n]`` in both the virtual output template and the original file.
     """
-    section, i, n, f_s = self.cur_section
+    if m == n :
+      raise self.MatchWithZeroLines('A placeholder was create with n == 0')
+    section, i, b_m, b_n, cb_begin = self.cur_section
     end = len(self.lines)
-    if f_s is None :
-      f_s = self.matcher(self.lines[i:i+n], begin=True)
-    if f is None :
-      f = self.matcher(self.lines[end-n:end], begin=False)
-    section.begin = f_s
-    section.end = f
+    if cb_begin is None :
+      cb_begin = self.matcher(self.lines[i + b_m : i + b_n], offset=b_m)
+    if cb is None :
+      cb = self.matcher(self.lines[end + m : end + n], offset=m)
+    section.begin = cb_begin
+    section.end = cb
     section.begin_line = i
     section.end_line = end
     self.cur_section = None
     self.sections.append(section)
   
-  def placeholder(self, name, n=1, f=None):
+  def placeholder(self, name, m=-1, n=0, cb=None):
     """
     Defines a placeholder
+    
     @param name : name of the placeholder
-    @param f : ``f`` is a calback function ``f(lines, i)`` where ``lines`` is a list of the lines in the original file. The function should return true if it matches.
+    @param cb : ``cb`` is a calback function ``cb(lines, i)`` where ``lines`` is a list of the lines in the original file, and ``i`` is the index of the current line being checked. The function should return true if the section should be inserted between lines[i] and lines[i+1]
 
-    @param n : If ``f`` is None, Then the ``n`` previous lines in the "virtually" outputed template (as if it were run for the first time) will be the line to match exactly in the original file to tag the placeholder.
+    @param n
+    @param m : If ``cb`` is None, Then the placeholder will try to match ``lines[cur + m : cur + n]`` in both the virtual output template and the original file.
     """
+    if m == n :
+      raise self.MatchWithZeroLines('A placeholder was create with n == 0')
     if self.cur_section is not None :
       raise OutStream.AlreadyInSectionError("A placeholder cannot be placed inside a section")
-    self.sections.append((name, len(self.lines), n, f))
+    self.sections.append((name, len(self.lines), m, n, cb))
   
   def write(self, s):
     self.lines.extend(s.split('\n')[:-1])
@@ -226,10 +306,33 @@ class OutStream(object):
   def getvalue(self):
     return "\n".join(self.lines) + '\n'
 
-  def newPlacehoder(self, name, i, n, f, /, keep_only_sections):
-    if keep_only_sections :
-      raise self.PlaceholderWithKeepOnlySections("No placeholder can be set if `keep_only_sections` is True") 
-    return self.Placeholder(name, f if f else self.matcher(self.lines[i:i+n], begin=True), i)
+  def newPlacehoder(self, name, i, m, n, cb, /, keep_only_sections):
+    return self.Placeholder(name, cb if cb else self.matcher(self.lines[i + m : i + n], m), i)
+
+  class ContextFactory(object):
+    """
+    Used to provide the `ctx()` callback to overwrite callbacks
+    """
+    def __init__(self, orig, virt, keep_only_sections):
+      self.orig = orig
+      self.virt = virt
+      self.section = None
+      self.keep_only_sections = keep_only_sections
+
+    def ctx(self):
+      return C(
+        o=C(
+          lines = self.orig,
+          sec_b = self.section.ori_begin_line,
+          sec_e = self.section.ori_end_line,
+        ),
+        v=C(
+          lines = self.virt,
+          sec_b = self.section.begin_line,
+          sec_e = self.section.end_line,
+        ),
+        keep_only_sections=self.keep_only_sections,
+      )
   
   def end(self, out_p, use_sections, keep_only_sections):
     """
@@ -243,34 +346,54 @@ class OutStream(object):
     with open(out_p) as f :
       lines = [ l[:-1] for l in f ]
     
-    candidate_sections = [ s if isinstance(s, self.Section) else self.newPlacehoder(*s, keep_only_sections=keep_only_sections) for s in self.sections ]
+    if keep_only_sections :
+      candidate_sections = [ s for s in self.sections if isinstance(s, self.Section) ]
+    else :
+      candidate_sections = [ s if isinstance(s, self.Section) else self.newPlacehoder(*s, keep_only_sections=keep_only_sections) for s in self.sections ]
     
     sections = []
     # match sections in any order
-    I = iter(range(len(lines)))
-    for i in I:
-      j, s = next(
-        (
+    i = 0
+    while i < len(lines) :
+      it = iter(
           (j, s) for j, s in enumerate(candidate_sections) if s.begin(lines, i)
-        ),
-        (None, None)
       )
-      if j is not None :
+      j, s = next(it, (None, None))
+      while j is not None :
         s.ori_begin_line = i
-        candidate_sections.pop(j)
-        for i in I :
-          if s.end(lines, i) :
-            s.ori_end_line = i
+        for i2 in range(i, len(lines) + 1) :
+          if s.end(lines, i2) :
+            s.ori_end_line = i2
             sections.append(s)
+            candidate_sections.pop(j)
+            i = i2
             break
+        else :
+          # The section did not end... Try next the one
+          j, s = next(it, (None, None))
+          continue
+        break
+      else :
+        # We tried all the sections without success... step.
+        i += 1
+
     # Remaining sections are the one not matched
     pldict = { pl.name : pl for pl in sections if isinstance(pl, self.Placeholder) }
-    
 
+    contextFactory = self.ContextFactory(lines, self.lines, keep_only_sections)
+    ctx = contextFactory.ctx
+    
     if keep_only_sections :
       # Replace sections in template with original file ones, except if overwrite
-      for s in reversed(self.sections) :
-        if not s.overwrite :
+      for s in reversed(sections) :
+        if callable(s.overwrite) :
+          contextFactory.section = s
+          self.lines[s.begin_line:s.end_line] = s.overwrite(
+            lines[s.ori_begin_line:s.ori_end_line],
+            self.lines[s.begin_line:s.end_line],
+            ctx
+          )
+        elif s.overwrite != True : # Could be None
           self.lines[s.begin_line:s.end_line] = lines[s.ori_begin_line:s.ori_end_line]
     else:
       # Merge placeholders and sections
@@ -280,18 +403,26 @@ class OutStream(object):
         pl = pldict.get(s.placeholder)
         if pl is not None :
           pl.sections.append(s)
-      # Replace sections in original file file with template ones except if overwrite False (in this case, only add at placeholder if not present)
-      
+          
+      # Replace sections in original file with template ones except if overwrite False (in this case, only add at placeholder if not present)
       for e in reversed(sections) :
         if isinstance(e, self.Section) :
           s = e
-          if s.overwrite :
+          if callable(s.overwrite) :
+            contextFactory.section = s
+            lines[s.ori_begin_line:s.ori_end_line] = s.overwrite(
+              lines[s.ori_begin_line:s.ori_end_line],
+              self.lines[s.begin_line:s.end_line],
+              ctx
+            )
+          elif s.overwrite != False : # Could be None
             lines[s.ori_begin_line:s.ori_end_line] = self.lines[s.begin_line:s.end_line]
         elif isinstance(e, self.Placeholder) :
           pl = e
           lines[pl.ori_begin_line:pl.ori_begin_line] = sum(( self.lines[s.begin_line:s.end_line] for s in pl.sections ), [])
       # Assign original file lines
       self.lines = lines
+  
   def close(self):
     self.lines.clear()
     self.sections.clear()
@@ -299,6 +430,41 @@ class OutStream(object):
     self._placeholders.clear()
     self.cur_section = None
 
+def extractHelpFromLocals(loc):
+  return next(( h for k in ('__doc__', 'help') if (h := loc.get(k)) ), 'No help provided for this template' ) 
+  
+
+def findTemplates(d: Path, root: Path):
+  if (d / 'root').exists() :
+    yield d.relative_to(root)
+  else :
+    for c in d.iterdir() :
+      if c.is_dir() :
+        yield from findTemplates(c, root)
+      else :
+        yield c.relative_to(root)
+      
+tempinySyntaxRegex = re.compile(r'^(\s*\S+)\s+\#\s+(\S+)__skbs_template__(\S+)\s*$')
+
+def tempinyFromLine(l):
+  m = tempinySyntaxRegex.fullmatch(l)
+  if not m :
+    return None
+  return Tempiny(stmt_line_start=m[1], begin_expr=m[2], end_expr=m[3])
+
+def tempinyFromIterable(iterable):
+  # New version support template file describing themselves their syntax...
+  it = iter(iterable)
+  first_line = next(it)
+  new_it = chain((first_line,), it)
+  return tempinyFromLine(first_line), new_it
+
+def _get(d, k, default):
+  v = d.get(k, None)
+  if v is None :
+    return default, False
+  return v, True
+  
 
 class Backend(object):
   OPT_PREFIX = '_opt.'
@@ -325,8 +491,8 @@ class Backend(object):
     default_dir = self.config.template_dir/'default/templates'
     user_dir = self.config.template_dir/'templates'
     return (
-      [ p.name for p in default_dir.iterdir()] if default_dir.is_dir() else [],
-      [ p.name for p in user_dir.iterdir()] if user_dir.is_dir() else []
+      list(findTemplates(default_dir, default_dir)) if default_dir.is_dir() else [],
+      list(findTemplates(user_dir, user_dir)) if user_dir.is_dir() else []
     )
 
   def installDefaultTemplates(self, symlink=False):
@@ -372,6 +538,16 @@ class Backend(object):
       remove_tree(str(dest))
     return dest
 
+  def createDummyPluginModule(self, path, g):
+    path = path.parent
+    g.__package__ = '__skbs_plugin__' + str(path).replace('/', '_').replace('.','_')
+    m = C(
+      __name__ = g.__package__,
+      __path__ = [str(path)],
+      __package__ = '',
+    )
+    sys.modules[g.__package__] = m
+
   def parsePlugin(self, path, args, dest, ask_help):
     tempiny = None
     plugin = None
@@ -390,20 +566,24 @@ class Backend(object):
       Tempiny=Tempiny,
       invokeTemplate=invokeTemplate,
       dest=dest if not ask_help else None,
+      parseCmd=OptionParser(args, plugin),
     )
     if path.is_file() :
       # source plugin.py if one
       with path.open('r') as f :
         obj = compile(f.read(), path, 'exec')
+      self.createDummyPluginModule(path, g)
       try:
         exec(obj, g.asDict(), g)
       except EndOfPlugin:
         pass
-    help = next(( h for k in ('__doc__', 'help') if (h := g.get(k)) ), 'No help provided for this template' ) 
+    help = extractHelpFromLocals(g)
     if ask_help :
       raise PluginError(help)
     conf = g.get('conf')
-    plugin = g.get('plugin')
+    plugin = g.get('plugin', None)
+    if plugin is None :
+      plugin = g.get('_p')
     return conf, plugin, help
 
   def parseConf(self, conf):
@@ -415,18 +595,19 @@ class Backend(object):
       tempiny_l = [ (pattern, Tempiny(**c)) for pattern, c in conf.tempiny ]
     else:
       tempiny_l = [ ('*', Tempiny()) ]
+    
     file_name_parser = FileNameParser(
-      conf.get('opt_prefix', self.OPT_PREFIX),
-      conf.get('force_prefix', self.FORCE_PREFIX),
-      conf.get('raw_prefix', self.RAW_PREFIX),
-      conf.get('template_prefix', self.TEMPLATE_PREFIX),
+      _get(conf, 'opt_prefix', self.OPT_PREFIX),
+      _get(conf, 'force_prefix', self.FORCE_PREFIX),
+      _get(conf, 'raw_prefix', self.RAW_PREFIX),
+      _get(conf, 'template_prefix', self.TEMPLATE_PREFIX),
       conf.get('dir_template_filename', None),
     )
     include_dirname = conf.get('include_dirname', self.INCLUDE_DIRNAME)
     pathmod_filename = conf.get('pathmod_filename', self.PATHMOD_FILENAME)
     return tempiny_l, file_name_parser, include_dirname, pathmod_filename
   
-  def findTemplate(self, template):
+  def findTemplate(self, template, single_file_authorized=False):
     """
     Find `template`. If `template` starts with a '@', then search in globally installed template.
     Else, if the path exists and point to a directoryn, return this directory.
@@ -439,14 +620,18 @@ class Backend(object):
         p = self.config.template_dir/'default/templates'/template[1:]
     else:
       p = Path(template)
+    if single_file_authorized and p.is_file() :
+      return p
     if not p.is_dir() :
-      raise FileNotFoundError(p)
+      if not p.is_file() :
+        raise FileNotFoundError(p)
+      return p
     return p
 
   @staticmethod
   def getFirstMatch(l, path):
     """
-    Get the second item of the first match on the frist item in a list of couples
+    Get the second item of the first match on the first item in a list of couples
     """
     return next(obj for glob_pat, obj in l if path.match(glob_pat))
   
@@ -490,14 +675,24 @@ class Backend(object):
     _locals.dest = out_p
     _locals.parent = out_p.parent if out_p is not None else None
     _locals.sls, _locals.be, _locals.ee = tempiny.conf
-    template = tempiny.compile(in_f, filename=in_p)
+    template = tempiny.compile(in_f, filename=in_p, add_to_linecache=True)
     return template(out_f, _locals.asDict())
 
   @classmethod
-  def processFile(cls, in_p, out_p, is_opt, is_template, base_locals, tempiny_l, dest):
+  def processFile(cls, in_p, out_p, is_opt, is_template, base_locals, tempiny_l, dest, out_f=None):
+    tempiny = None
+    if is_template != False :
+      with in_p.open('r') as in_f :
+        tempiny, _ = tempinyFromIterable(in_f)
+      is_template = is_template or (tempiny is not None)
+
     if is_template :
       tmp_out_f = OutStream()
-      tempiny = cls.getFirstMatch(tempiny_l, out_p)
+      if tempiny is None :
+        if out_p :
+          tempiny = cls.getFirstMatch(tempiny_l, out_p)
+        else:
+          _, tempiny = tempiny_l[0]
       with in_p.open('r') as in_f :
         _locals = {
           **base_locals,
@@ -509,30 +704,39 @@ class Backend(object):
         if exc :
           try :
             raise exc
-          except EndOfPlugin:
+          except EndOfTemplate:
             pass
+          except EndOfPlugin:
+            raise PluginError(extractHelpFromLocals(_locals))
           except ExcludeFile:
-            return
+            return _locals
       out_p = _locals.get('new_path', out_p)
       is_opt = _locals.get('is_opt', is_opt)
       if out_p is None :
-        return
+        if out_f :
+          out_f.write(tmp_out_f.getvalue())
+        return _locals
       out_p = dest / out_p
       if out_p.exists() :
         if is_opt :
-          return
+          return _locals
         else:
-          tmp_out_f.end(out_p, _locals.get('use_sections'), _locals.get('keep_only_sections', True))
+          tmp_out_f.end(out_p, _locals.get('use_sections'), _locals.get('keep_only_sections', False))
       out_p.parent.mkdir(parents=True, exist_ok=True)
-      with out_p.open('w') as out_f :
+      if out_f :
         out_f.write(tmp_out_f.getvalue())
+      else:
+        with out_p.open('w') as out_f :
+          out_f.write(tmp_out_f.getvalue())
       tmp_out_f.close()
+      return _locals
     else:
       out_p = dest / out_p
       if is_opt :
         if out_p.exists() :
-          return
+          return C()
       shutil.copyfile(in_p, out_p)
+      return C()
   @staticmethod
   def processDir(base_locals, in_p, out_p, file_name_parser):
     path = in_p / file_name_parser.dir_template_filename
@@ -544,42 +748,98 @@ class Backend(object):
     _locals.dest = out_p
     try:
       exec(obj, _locals.asDict(), _locals)
-    except EndOfPlugin:
+    except EndOfTemplate:
       pass
     except ExcludeFile:
       return None
     return _locals.get('new_path', out_p)
 
-  def execSingleFileTemplate(self, template_path : Path, dest : str, args, plugin = FlagPluginNotParsed):
+  def execSingleFileTemplate(self, template_path : Path, dest : str, args:list[str], out_f=None, plugin = C()):
     ask_help = (dest == '@help') or (args and args[0] == '--help')
-    try:
-      conf, plugin, help = self.parsePlugin(template_path / 'plugin.py', args, dest, ask_help)
-    except PluginError as err:
-      return False, err.help
-    dest = Path(dest)
-    
-    tempiny_l, file_name_parser, include_dirname, pathmod_filename = self.parseConf(conf)
+    if dest == '@' :
+      dest_name = None
+      dest_parent = Path()
+    else:
+      dest = Path(dest)
+      dest_name = Path(dest.name)
+      dest_parent = dest.parent
+
+    # Template as file
+    if template_path.is_file() :
+      import re
+      with open(template_path, 'r') as f :
+        l = next(f)
+      m = re.fullmatch(tempinySyntaxRegex, l)
+      if m :
+        conf = C(
+          tempiny = [
+            ('*', dict(stmt_line_start=m[1], begin_expr=m[2], end_expr=m[3]))
+          ]
+        )
+      else :
+        conf = C()
+        
+      tempiny_l, file_name_parser, include_dirname, pathmod_filename = self.parseConf(conf)
+        
+      base_locals = C(
+        args = args,
+        ask_help=ask_help,
+        C=C,
+        click=click,
+        invokeCmd = invokeCmd,
+        EndOfPlugin=EndOfPlugin,
+        PluginError=PluginError,
+        pluginError=pluginError,
+        dest=dest if not ask_help else None,
+        parseCmd=OptionParser(args, plugin),
+        inside_skbs_plugin=True,
+        plugin=plugin,
+        _p=plugin,
+        removePrefix=file_name_parser,
+        file_name_parser=file_name_parser,
+        exclude=exclude,
+        endOfTemplate=endOfTemplate,
+        invokeTemplate=self.invokeTemplate,
+      )
       
-    base_locals = C(
-      plugin=plugin,
-      _p=plugin,
-      C=C,
-      removePrefix=file_name_parser,
-      file_name_parser=file_name_parser,
-      exclude=exclude,
-      endOfTemplate=endOfTemplate,
-    )
-    base_locals.include = Include([template_path / '__include'], tempiny_l, base_locals, file_name_parser)
-    
-    self.processFile(template_path/'root', Path(dest.name), False, True, base_locals, tempiny_l, dest.parent)
-    return True, help
+      try:
+        _locals = self.processFile(template_path, dest_name, False, True, base_locals, tempiny_l, dest_parent, out_f=out_f)
+      except PluginError as err:
+        return False, err.help
+      return True, _locals.get('help')
+      
+    # Template as dir
+    else:
+      try:
+        conf, plugin, help = self.parsePlugin(template_path / 'plugin.py', args, dest, ask_help)
+      except PluginError as err:
+        return False, err.help
+      dest = Path(dest)
+      
+      tempiny_l, file_name_parser, include_dirname, pathmod_filename = self.parseConf(conf)
+        
+      base_locals = C(
+        plugin=plugin,
+        _p=plugin,
+        C=C,
+        removePrefix=file_name_parser,
+        file_name_parser=file_name_parser,
+        exclude=exclude,
+        endOfTemplate=endOfTemplate,
+        invokeTemplate=self.invokeTemplate,
+      )
+      base_locals.include = Include([template_path / '__include'], tempiny_l, base_locals, file_name_parser)
+      
+      self.processFile(template_path/'root', dest_name, False, True, base_locals, tempiny_l, dest_parent, out_f=out_f)
+      return True, help
     
   
-  def execTemplate(self, template_path : Path, dest : str, args):
-    from hl037utils.config import Config as C
+  def execTemplate(self, template_path : Path, dest : str, args, out_f=None):
     if not (template_path/'root').is_dir() :
-      return self.execSingleFileTemplate(template_path, dest, args)
+      return self.execSingleFileTemplate(template_path, dest, args, out_f=out_f)
       
+    if dest == '@' or out_f is not None:
+      return False, 'Stream output is not available as dest for multi-file plugins.'
     ask_help = (dest == '@help') or (args and args[0] == '--help')
     try:
       conf, plugin, help = self.parsePlugin(template_path / 'plugin.py', args, dest, ask_help)
@@ -600,6 +860,7 @@ class Backend(object):
       file_name_parser=file_name_parser,
       exclude=exclude,
       endOfTemplate=endOfTemplate,
+      invokeTemplate=self.invokeTemplate,
     )
     base_locals.include = Include(include_paths, tempiny_l, base_locals, file_name_parser)
     
@@ -645,6 +906,8 @@ class Backend(object):
         self.processFile(in_p, out_p, is_opt, is_template, base_locals, tempiny_l, dest)    
     return True, help
 
-  def invokeTemplate(self, template_name, dest, args):
-    self.execTemplate(self.findTemplate(template_name), dest, args)
+  def invokeTemplate(self, template_name, dest, args, out_f=None, single_file_authorized=None):
+    if single_file_authorized is None :
+      single_file_authorized = out_f is not None
+    return self.execTemplate(self.findTemplate(template_name, single_file_authorized=single_file_authorized), dest, args, out_f)
 
